@@ -5,20 +5,25 @@ package Tk::DirTree;
 #
 # Chris Dean <ctdean@cogit.com>
 
+use strict;
 use vars qw($VERSION);
-$VERSION = '4.006'; # $Id: //depot/Tkutf8/Tixish/DirTree.pm#6 $
+$VERSION = '4.020';
 
 use Tk;
 use Tk::Derived;
 use Tk::Tree;
 use Cwd;
 use DirHandle;
+use File::Spec qw();
 
 use base  qw(Tk::Derived Tk::Tree);
 use strict;
 
 Construct Tk::Widget 'DirTree';
 
+my $sep = $^O eq 'MSWin32' ? '\\' : '/';
+
+*_fs_encode = eval { require Encode; 1 } ? sub { Encode::encode("iso-8859-1", $_[0]) } : sub { $_[0] };
 
 sub Populate {
     my( $cw, $args ) = @_;
@@ -32,12 +37,14 @@ sub Populate {
         -directory      => [qw/SETMETHOD directory Directory ./],
         -value          => '-directory' );
 
-    $cw->configure( -separator => '/', -itemtype => 'imagetext' );
+    $cw->configure( -separator => $sep,
+		    -itemtype => 'imagetext',
+		  );
 }
 
 sub DirCmd {
     my( $w, $dir, $showhidden ) = @_;
-
+    $dir .= $sep if $dir =~ /^[a-z]:$/i and $^O eq 'MSWin32';
     my $h = DirHandle->new( $dir ) or return();
     my @names = grep( $_ ne '.' && $_ ne '..', $h->read );
     @names = grep( ! /^[.]/, @names ) unless $showhidden;
@@ -50,15 +57,16 @@ sub fullpath
 {
  my ($path) = @_;
  my $cwd = getcwd();
- if (chdir($path))
+ if (CORE::chdir($path))
   {
    $path = getcwd();
-   chdir($cwd) || die "Cannot cd back to $cwd:$!";
+   CORE::chdir($cwd) || die "Cannot cd back to $cwd:$!";
   }
  else
   {
    warn "Cannot cd to $path:$!"
   }
+ $path = File::Spec->canonpath($path);
  return $path;
 }
 
@@ -75,7 +83,7 @@ sub set_dir {
     my( $w, $val ) = @_;
     my $fulldir = fullpath( $val );
 
-    my $parent = '/';
+    my $parent = $sep;
     if ($^O eq 'MSWin32')
      {
       if ($fulldir =~ s/^([a-z]:)//i)
@@ -86,10 +94,10 @@ sub set_dir {
     $w->add_to_tree( $parent, $parent)  unless $w->infoExists($parent);
 
     my @dirs = ($parent);
-    foreach my $name (split( /[\/\\]/, $fulldir )) {
+    foreach my $name (split( /\Q$sep\E/, $fulldir )) {
         next unless length $name;
         push @dirs, $name;
-        my $dir = join( '/', @dirs );
+	my $dir = File::Spec->catfile( @dirs );
         $w->add_to_tree( $dir, $name, $parent )
             unless $w->infoExists( $dir );
         $parent = $dir;
@@ -98,16 +106,17 @@ sub set_dir {
     $w->OpenCmd( $parent );
     $w->setmode( $parent, 'close' );
 }
+*chdir = \&set_dir;
 
 
 sub OpenCmd {
     my( $w, $dir ) = @_;
 
     my $parent = $dir;
-    $dir = '' if $dir eq '/';
     foreach my $name ($w->dirnames( $parent )) {
         next if ($name eq '.' || $name eq '..');
-        my $subdir = "$dir/$name";
+        my $subdir = File::Spec->catfile( $dir, $name );
+	$subdir = _fs_encode($subdir);
         next unless -d $subdir;
         if( $w->infoExists( $subdir ) ) {
             $w->show( -entry => $subdir );
@@ -122,14 +131,20 @@ sub OpenCmd {
 sub add_to_tree {
     my( $w, $dir, $name, $parent ) = @_;
 
-    my $image = $w->Getimage( $w->cget('-image') );
+    my $dir8 = _fs_encode($dir);
+    my $image = $w->cget('-image');
+    if ( !UNIVERSAL::isa($image, 'Tk::Image') ) {
+	$image = $w->Getimage( $image );
+    }
     my $mode = 'none';
     $mode = 'open' if $w->has_subdir( $dir );
 
     my @args = (-image => $image, -text => $name);
     if( $parent ) {             # Add in alphabetical order.
         foreach my $sib ($w->infoChildren( $parent )) {
-            if( $sib gt $name ) {
+	    use if $^O ne 'MSWin32', "locale"; # dumps core under Windows under some (japanese?) locales, see http://www.nntp.perl.org/group/perl.cpan.testers/2008/11/msg2550386.html
+	    my $sib8 = _fs_encode($sib);
+	    if ($sib8 gt $dir8) {
                 push @args, (-before => $sib);
                 last;
             }
@@ -145,7 +160,7 @@ sub has_subdir {
     foreach my $name ($w->dirnames( $dir )) {
         next if ($name eq '.' || $name eq '..');
         next if ($name =~ /^\.+$/);
-        return( 1 ) if -d "$dir/$name";
+        return( 1 ) if -d File::Spec->catfile( $dir, $name );
     }
     return( 0 );
 }
@@ -156,6 +171,80 @@ sub dirnames {
     return( @names );
 }
 
+{
+    package Tk::DirTreeDialog;
+    use base qw(Tk::Toplevel);
+    Construct Tk::Widget 'DirTreeDialog';
+
+    sub Populate {
+	my($w, $args) = @_;
+	$w->{curr_dir} = delete $args->{-initialdir};
+	if (!defined $w->{curr_dir}) {
+	    require Cwd;
+	    $w->{curr_dir} = Cwd::cwd();
+	}
+	if (defined $args->{-mustexist}) {
+	    die "-mustexist is not yet implemented";
+	}
+	my $title = $args->{-title} || "Choose directory:";
+	delete $args->{-popover};
+
+	$w->title($title);
+	$w->{ok} = 0; # flag: "1" means OK, "-1" means cancelled
+
+	# Create Frame widget before the DirTree widget, so it's always visible
+	# if the window gets resized.
+	my $f = $w->Frame->pack(-fill => "x", -side => "bottom");
+
+	my $d;
+	$d = $w->Scrolled('DirTree',
+			  -scrollbars => 'osoe',
+			  -width => 35,
+			  -height => 20,
+			  -selectmode => 'browse',
+			  -exportselection => 1,
+			  -browsecmd => sub {
+			      $w->{curr_dir} = shift;
+			  },
+
+			  # With this version of -command a double-click will
+			  # select the directory
+			  -command   => sub { $w->{ok} = 1 },
+
+			  # With this version of -command a double-click will
+			  # open a directory. Selection is only possible with
+			  # the Ok button.
+			  #-command   => sub { $d->opencmd($_[0]) },
+			 )->pack(-fill => "both", -expand => 1);
+	# Set the initial directory
+	$d->set_dir($w->{curr_dir});
+
+	$f->Button(-text => 'Ok',
+		   -command => sub { $w->{ok} =  1 })->pack(-side => 'left');
+	$f->Button(-text => 'Cancel',
+		   -command => sub { $w->{ok} = -1 })->pack(-side => 'left');
+	$w->OnDestroy(sub { $w->{ok} = -1 });
+    }
+
+    sub Show {
+	my $w = shift;
+	my $old_focus = $w->focusSave;
+	my $old_grab = $w->grabSave;
+	Tk::catch {
+	    $w->grab;
+	};
+	$w->waitVariable(\$w->{ok});
+	my $ret = $w->{ok} == 1 ? $w->{curr_dir} : undef;
+	$w->grabRelease if Tk::Exists($w);
+	&$old_focus;
+	&$old_grab;
+	$w->destroy if Tk::Exists($w);
+	$ret;
+    }
+}
+
+1;
+
 __END__
 
 #  Copyright (c) 1996, Expert Interface Technologies
