@@ -1,4 +1,4 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2003 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::MMutil;
@@ -9,7 +9,7 @@ use Carp;
 use File::Basename;
 
 use vars qw($VERSION);
-$VERSION = '4.012'; # $Id: //depot/Tkutf8/Tk/MMutil.pm#12 $
+$VERSION = '4.023'; # was: sprintf '4.%03d', q$Revision: #21 $ =~ /\D(\d+)\s*$/;
 
 # warn __FILE__." $VERSION\n";
 
@@ -20,7 +20,7 @@ use vars qw($IsWin32);
 
 *IsWin32 = \$main::IsWin32;
 $IsWin32 = ($^O eq 'MSWin32' || $Config{'ccflags'} =~ /-D_?WIN32_?/)
-           unless defined $IsWin32;
+	   unless defined $IsWin32;
 
 @MYEXPORT = qw(pasthru perldepend cflags const_config constants installbin c_o xs_o makefile manifypods);
 
@@ -60,13 +60,13 @@ sub mTk_postamble
  my ($self) = @_;
  my $dep = "config :: \$(C_FILES) \$(H_FILES)\n\t$self->{NOECHO}\$(NOOP)\n";
  my $mTk = $self->{'MTK'};
- $dep .= "# Begin Munging dependancies\n";
+ $dep .= "# Begin Munging dependencies\n";
  foreach my $file (sort keys %$mTk)
   {
    $dep .= "$file : ".$mTk->{$file}." \$(TKDIR)/pTk/Tcl-pTk\n";
    $dep .= "\t\$(PERL) \$(TKDIR)/pTk/Tcl-pTk ".$mTk->{$file}." $file\n";
   }
- $dep .= "# End Munging dependancies\n\n";
+ $dep .= "# End Munging dependencies\n\n";
  return $dep;
 }
 
@@ -163,34 +163,34 @@ sub relpath
       {
        my $depth = reverse($1);
        if ($depth)
-        {
-         $depth =~ s,[^/\\]+,..,g;
-        }
+	{
+	 $depth =~ s,[^/\\]+,..,g;
+	}
        else
-        {
-         $depth = '.' ;
-        }
+	{
+	 $depth = '.' ;
+	}
        $depth =~ s,[/\\]+$,,;
        $base =~ s,^[/\\]+,,;
        $depth .= "/$base" if ($base);
        if (-e $depth)
-        {
-         # print "$path is $depth from $here\n";
-         return $depth;
-        }
+	{
+	 # print "$path is $depth from $here\n";
+	 return $depth;
+	}
        else
-        {
-         warn "Cannot find $depth\n";
-        }
+	{
+	 warn "Cannot find $depth\n";
+	}
       }
      else
       {
        unless(exists $visited{$here})
-        {
-         $visited{$here} = 1;
-         warn "$here does not start with $dir\n";
-         warn "i.e. building outside Tk itself\n";
-        }
+	{
+	 $visited{$here} = 1;
+	 warn "$here does not start with $dir\n";
+	 warn "i.e. building outside Tk itself\n";
+	}
       }
     }
    else
@@ -220,8 +220,13 @@ sub pasthru
  my $str = $self->MM::pasthru;
  if ($str =~ s/^\s+INC=.*\n//m)
   {
-   return "\n#Do NOT pasthru INC for Tk - it is computed by subdir\n".$str;
+   $str = "# - Do NOT pasthru INC for Tk - it is computed by subdir\n$str"
   }
+ if ($str =~ s/\bLIB="\$\(LIB\)"//)
+  {
+   $str = qq[# - Drop LIB="\$(LIB)" - not used\n$str];
+  }
+ $str = "#Tk::MMutil pasthru\n$str";
  return $str;
 }
 
@@ -254,10 +259,12 @@ sub perldepend
       {
        push(@def,qw(-D__WIN32__)) unless $self->{'DEFINE'} =~ /-D__WIN32__/;
        push(@def,qw(-DWIN32)) if $self->{'NAME'} eq 'Tk::pTk';
+       push(@def,qw(-dWINARCH_MSWIN32));
       }
      elsif ($win_arch eq 'x')
       {
        push(@def,qw(-U_WIN32));
+       push(@def,qw(-DWINARCH_X));
       }
     }
    foreach (@inc)
@@ -286,7 +293,22 @@ sub const_config
    $self->{'LDFLAGS'} =~ s/-(debug|pdb:\w+)\s+//g;
    $self->{'LDDLFLAGS'} =~ s/-(debug|pdb:\w+)\s+//g;
   }
- elsif ($^O =~ /(openbsd)/i)
+ elsif ($^O eq 'darwin' )
+  {
+   $self->{'LDDLFLAGS'} =~ s/-flat_namespace//;
+   $self->{'LDDLFLAGS'} =~ s/-undefined\s+suppress//;
+   if ( -e "$Config{'archlib'}/CORE/$Config{'libperl'}" ) {
+    $self->{'LDDLFLAGS'} .= " -L\${PERL_ARCHLIB}/CORE -lperl ";
+   }
+   elsif ( -e "/System/Library/Perl/darwin/CORE/libperl.dylib" ) {
+    $self->{'LDDLFLAGS'} .= " -L/System/Library/Perl/darwin/CORE -lperl ";
+   }
+   else {
+    warn "Can't find libperl.dylib";
+   }
+   $self->{'LDFLAGS'} =~ s/-flat_namespace//;
+   $self->{'LDFLAGS'} =~ s/-undefined\s+suppress//;
+  } elsif ($^O =~ /(openbsd)/i)
   {
    # -Bforcearchive is bad news for Tk - we don't want all of libpTk.a in all .so-s.
    $self->{'LDDLFLAGS'} =~ s/-Bforcearchive\s*//g;
@@ -346,15 +368,15 @@ sub manifypods
  local $_ = $self->MM::manifypods;
  if ($] >= 5.00565)
   {
-   s/(POD2MAN_EXE.*pod2man)/$1 --center "perl\/Tk Documentation" --release "Tk\$(VERSION)"/;
+   s/(POD2MAN_EXE.*pod2man.*)/$1 --center "perl\/Tk Documentation" --release "Tk\$(VERSION)"/;
   }
  elsif ($] >= 5.003)
   {
-   s/(POD2MAN_EXE.*pod2man)/$1 -center "perl\/Tk Documentation" -release "Tk\$(VERSION)"/;
+   s/(POD2MAN_EXE.*pod2man.*)/$1 -center "perl\/Tk Documentation" -release "Tk\$(VERSION)"/;
   }
  else
   {
-   s/(POD2MAN_EXE.*pod2man)/$1 -center \\"perl\/Tk Documentation\\" -release \\"Tk\$(VERSION)\\"/;
+   s/(POD2MAN_EXE.*pod2man.*)/$1 -center \\"perl\/Tk Documentation\\" -release \\"Tk\$(VERSION)\\"/;
   }
  s/\bpod::/Tk::/mg;
  s/\bpTk:://mg;
@@ -492,7 +514,7 @@ sub TkExtMakefile
  $att{'macro'} = {} unless (exists $att{'macro'});
  $att{'macro'}{'TKDIR'} = $tk;
  my @opt = ('VERSION'     => $Tk::Config::VERSION,
-            'XS_VERSION'  => $Tk::Config::VERSION);
+	    'XS_VERSION'  => $Tk::Config::VERSION);
  push(@opt,'clean' => {} ) unless (exists $att{'clean'});
  $att{'clean'}->{FILES} = '' unless (exists $att{'clean'}->{FILES});
  $att{'clean'}->{FILES} .= ' *.bak';
@@ -507,9 +529,16 @@ sub TkExtMakefile
    if (delete $att{'dynamic_ptk'})
     {
      push(@opt,
-          'MYEXTLIB' => "$ptk/libpTk\$(LIB_EXT)",
+	  'MYEXTLIB' => "$ptk/libpTk\$(LIB_EXT)",
 #         'dynamic_lib' => { INST_DYNAMIC_DEP => "$ptk/libpTk\$(LIB_EXT)" }
-         );
+	 );
+    }
+   # Several loadable widgets use things from -lm
+   # if platform does not have a shared -lm need to link against it
+   if ($Config{libs} =~/-lm\b/)
+    {
+     my $libs = $att{'LIBS'}->[0];
+     $att{'LIBS'}->[0] = "$libs -lm" unless $libs =~ /-lm\b/;
     }
    if ($IsWin32 && $Config{'cc'} =~ /^bcc/)
     {
@@ -521,7 +550,7 @@ sub TkExtMakefile
     {
      my $base  = $Config{'libpth'};
      $base =~ s#lib$#i386-mingw32/lib#;
-     my $extra = "-L$base -lcomdlg32 -lgdi32";
+     my $extra = "-L$base -limm32 -lcomctl32 -lcomdlg32 -lgdi32";
      my $libs = $att{'LIBS'}->[0];
      $att{'LIBS'}->[0] = "$extra $libs";
     }
@@ -533,15 +562,15 @@ sub TkExtMakefile
      push(@opt,'LD' => 'gcc -shared');
      if ($win_arch eq 'MSWin32')
       {
-       my $extra = "-lcomdlg32 -lgdi32";
+       my $extra = "-L/lib/w32api -limm32 -lcomctl32 -lcomdlg32 -lgdi32";
        my $libs = $att{'LIBS'}->[0];
        $att{'LIBS'}->[0] = "$extra $libs";
-       $att{'DEFINE'} .= ' -D__WIN32__';
+       $att{'DEFINE'} .= ' -D__WIN32__ -D_WIN32 -DWINARCH_MSWIN32';
        $att{'DEFINE'} .= ' -DWIN32' if($att{'NAME'} eq 'Tk::pTk');
       }
      elsif ($win_arch eq 'x')
       {
-       $att{'DEFINE'} .= ' -U_WIN32';
+       $att{'DEFINE'} .= ' -U_WIN32 -DWINARCH_X';
       }
     }
    if (delete $att{'ptk_include'})
