@@ -3,6 +3,7 @@
 
 #define STATIC_BUILD
 
+
 #ifdef bool
 #undef bool
 #endif
@@ -10,6 +11,12 @@
 #include "tkConfig.h"
 #define TCL_NO_DEPRECATED
 
+#if !defined(WIN32) && defined(USE_XFT_FONTS)
+#ifndef TCL_UTF_MAX
+#define TCL_UTF_MAX 13
+#endif
+#endif
+
 #define USE_TCLALLOC 1
 #define TCL_MEM_DEBUG
 #define USE_COMPAT_CONST
@@ -48,9 +55,6 @@ typedef void *TclHandle;
 extern Tcl_ObjType	tclIntType;
 #endif
 
-#ifndef Arg
-#define Arg Tcl_Obj *
-#endif
 #ifndef Var
 #define Var Tcl_Obj *
 #endif
@@ -85,17 +89,8 @@ EXTERN int  LangCmpOpt  _ANSI_ARGS_((CONST char *opt,CONST char *arg,size_t leng
 EXTERN void LangFreeArg _ANSI_ARGS_((Tcl_Obj *,Tcl_FreeProc *freeProc));
 EXTERN Tcl_Obj *LangCopyArg _ANSI_ARGS_((Tcl_Obj *));
 
-#define panic Tcl_Panic
-
-
 EXTERN int  LangNull _ANSI_ARGS_((Tcl_Obj *));
 
-/* Used to default Menu variable to the label
-   TCL just strdup's the string so it can be ckfree'ed
-*/
-
-EXTERN void LangExit _ANSI_ARGS_((int));
-
 EXTERN void		TclpGetTime  _ANSI_ARGS_((Tcl_Time *time));
 EXTERN void		TclpAsyncMark _ANSI_ARGS_((Tcl_AsyncHandler async));
 
@@ -103,9 +98,13 @@ EXTERN void Lang_SetErrorCode _ANSI_ARGS_((Tcl_Interp *interp,char *code));
 EXTERN char *Lang_GetErrorCode _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN char *Lang_GetErrorInfo _ANSI_ARGS_((Tcl_Interp *interp));
 
+/* Old-config handler for variables */
 EXTERN int LangSaveVar _ANSI_ARGS_((Tcl_Interp *,Tcl_Obj *,Var *,int type));
 EXTERN void LangFreeVar _ANSI_ARGS_((Var));
 
+/* New-config handler for objects, variables and callbacks */
+EXTERN int LangConfigObj _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj **save,
+				      Tcl_Obj *obj, int type));
 
 EXTERN int LangEventHook _ANSI_ARGS_((int flags));
 EXTERN void Lang_BuildInImages _ANSI_ARGS_((void));
@@ -164,7 +163,18 @@ EXTERN void		Lang_UntraceVar _ANSI_ARGS_((Tcl_Interp * interp,
 EXTERN int		Tk_PropertyCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, Tcl_Obj **objv));
 
+struct TkFontAttributes;
 
+EXTERN unsigned int     LangFontRank _ANSI_ARGS_((unsigned int suggested,
+			    int ch, CONST char *gotName,
+			    CONST char *wantFoundary,
+			    CONST struct TkFontAttributes *wantAttrib,
+			    CONST char *wantEncoding,
+			    CONST char *gotFoundary,
+			    CONST struct TkFontAttributes *gotAttrib,
+			    CONST char *gotEncoding));
+
+EXTERN void		Lang_FreeRegExp _ANSI_ARGS_((Tcl_RegExp re));
 
 EXTERN long Lang_OSHandle _ANSI_ARGS_((int fd));
 #define TK_LIBRARY LangLibraryDir()
